<?php

declare(strict_types=1);

namespace Reloadify\Controller;

use Reloadify\Service\AbandonedCartService;
use Sensio\Bundle\FrameworkExtraBundle\Configuration\Cache;
use Shopware\Core\Checkout\Cart\Cart;
use Shopware\Core\Checkout\Cart\CartPersisterInterface;
use Shopware\Core\Checkout\Cart\SalesChannel\CartService;
use Shopware\Core\Framework\Routing\Annotation\RouteScope;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Shopware\Storefront\Controller\StorefrontController;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\Routing\Annotation\Route;

/**
 * @RouteScope(scopes={"storefront"})
 */
class AbandonedCartController extends StorefrontController
{
    protected AbandonedCartService $abandonedCartService;
    protected CartService $cartService;
    protected CartPersisterInterface $cartPersister;

    public function __construct(
        AbandonedCartService   $abandonedCartService,
        CartService            $cartService,
        CartPersisterInterface $cartPersister
    )
    {
        $this->abandonedCartService = $abandonedCartService;
        $this->cartService = $cartService;
        $this->cartPersister = $cartPersister;
    }

    /**
     * Claim a cart by its id in reloadify_abandoned_cart e.g. '635948eb99154696a9a8a0532a6096bb'
     * @Route("claim-cart/{id}", name="reloadify.frontend.claim.cart", methods={"GET"})
     * @Cache(mustRevalidate=true)
     * @param Request $request
     * @param SalesChannelContext $context
     * @param string $id
     * @return Response
     */
    public function claimCart(Request $request, SalesChannelContext $context, string $id): Response
    {
        //Get abandoned cart from database with id
        $abandonedCartEntity = $this->abandonedCartService->getAbandonedCartWithId($id, $context->getContext());

        if (empty($abandonedCartEntity)) {
            $this->addFlash(self::WARNING, $this->trans('reloadify.checkout.noCartFoundMessage'));
            return $this->forwardToRoute('frontend.home.page', ['success' => false]);
        }

        /** @var Cart $cart */
        $cart = unserialize($abandonedCartEntity->getCart());

        $cart->setToken($context->getToken());
        $this->abandonedCartService->setAbandonedCartToken($id,$context->getToken(),$context->getContext());
        $this->cartService->setCart($cart);
        $this->cartService->recalculate($cart, $context);
        $this->addFlash(self::INFO, $this->trans('reloadify.checkout.cartAppliedMessage'));

        return $this->forwardToRoute('frontend.checkout.cart.page');
    }
}
