<?php

namespace Reloadify\Service;

use Reloadify\Event\AbandonedCartDeleteEvent;
use Shopware\Core\Checkout\Cart\Cart;
use Shopware\Core\Checkout\Cart\CartPersisterInterface;
use Shopware\Core\System\SalesChannel\SalesChannelContext;
use Symfony\Contracts\EventDispatcher\EventDispatcherInterface;

class CartPersisterDecorator implements CartPersisterInterface
{
    private CartPersisterInterface $cartPersister;
    private EventDispatcherInterface $eventDispatcher;

    public function __construct(CartPersisterInterface $cartPersister, EventDispatcherInterface $eventDispatcher)
    {
        $this->cartPersister = $cartPersister;
        $this->eventDispatcher= $eventDispatcher;
    }


    public function load(string $token, SalesChannelContext $context): Cart
    {
        return $this->cartPersister->load($token, $context);
    }

    public function save(Cart $cart, SalesChannelContext $context): void
    {
        $this->cartPersister->save($cart, $context);
    }

    public function delete(string $token, SalesChannelContext $context): void
    {
        $this->eventDispatcher->dispatch(new AbandonedCartDeleteEvent($token,$context));
        $this->cartPersister->delete($token, $context);
    }
}
